@@ -20,13 +20,23 @@
 #'   y scale; if \code{"free_x"} their width will be proportional to the
 #'  length of the x scale; or if \code{"free"} both height and width will
 #'  vary.  This setting has no effect unless the appropriate scales also vary.
-#' @param labeller A function that takes two arguments (\code{variable} and
-#'   \code{value}) and returns a string suitable for display in the facet
-#'   strip. See \code{\link{label_value}} for more details and pointers
-#'   to other options.
+#' @param labeller A function that takes one data frame of labels and
+#'   returns a list or data frame of character vectors. Each input
+#'   column corresponds to one factor. Thus there will be more than
+#'   one with formulae of the type \code{~cyl + am}. Each output
+#'   column gets displayed as one separate line in the strip
+#'   label. This function should inherit from the "labeller" S3 class
+#'   for compatibility with \code{\link{labeller}()}. See
+#'   \code{\link{label_value}} for more details and pointers to other
+#'   options.
 #' @param as.table If \code{TRUE}, the default, the facets are laid out like
 #'   a table with highest values at the bottom-right. If \code{FALSE}, the
 #'   facets are laid out like a plot with the highest value at the top-right.
+#' @param switch By default, the labels are displayed on the top and
+#'   right of the plot. If \code{"x"}, the top labels will be
+#'   displayed to the bottom. If \code{"y"}, the right-hand side
+#'   labels will be displayed to the left. Can also be set to
+#'   \code{"both"}.
 #' @param shrink If \code{TRUE}, will shrink scales to fit output of
 #'   statistics, not raw data. If \code{FALSE}, will be range of raw data
 #'   before statistical summary.
@@ -35,108 +45,69 @@
 #'   will be shown, regardless of whether or not they appear in the data.
 #' @export
 #' @examples
-#' \donttest{
-#' p <- ggplot(mtcars, aes(mpg, wt)) + geom_point()
-#' # With one variable
-#' p + facet_grid(. ~ cyl)
-#' p + facet_grid(cyl ~ .)
+#' p <- ggplot(mpg, aes(displ, cty)) + geom_point()
 #'
-#' # With two variables
-#' p + facet_grid(vs ~ am)
-#' p + facet_grid(am ~ vs)
-#' p + facet_grid(vs ~ am, margins=TRUE)
+#' p + facet_grid(. ~ cyl)
+#' p + facet_grid(drv ~ .)
+#' p + facet_grid(drv ~ cyl)
 #'
 #' # To change plot order of facet grid,
 #' # change the order of variable levels with factor()
 #'
-#' set.seed(6809)
-#' diamonds <- diamonds[sample(nrow(diamonds), 1000), ]
-#' diamonds$cut <- factor(diamonds$cut,
-#'          levels = c("Ideal", "Very Good", "Fair", "Good", "Premium"))
-#'
-#' # Repeat first example with new order
-#' p <- ggplot(diamonds, aes(carat, ..density..)) +
-#'         geom_histogram(binwidth = 1)
-#' p + facet_grid(. ~ cut)
-#'
-#' qplot(mpg, wt, data=mtcars, facets = . ~ vs + am)
-#' qplot(mpg, wt, data=mtcars, facets = vs + am ~ . )
-#'
-#' # You can also use strings, which makes it a little easier
-#' # when writing functions that generate faceting specifications
-#' # p + facet_grid("cut ~ .")
-#'
-#' # see also ?plotmatrix for the scatterplot matrix
-#'
-#' # If there isn't any data for a given combination, that panel
-#' # will be empty
-#' qplot(mpg, wt, data=mtcars) + facet_grid(cyl ~ vs)
-#'
-# If you combine a facetted dataset with a dataset that lacks those
-# facetting variables, the data will be repeated across the missing
-# combinations:
-#' p <- qplot(mpg, wt, data=mtcars, facets = vs ~ cyl)
-#'
-#' df <- data.frame(mpg = 22, wt = 3)
-#' p + geom_point(data = df, colour="red", size = 2)
-#'
-#' df2 <- data.frame(mpg = c(19, 22), wt = c(2,4), vs = c(0, 1))
-#' p + geom_point(data = df2, colour="red", size = 2)
-#'
-#' df3 <- data.frame(mpg = c(19, 22), wt = c(2,4), vs = c(1, 1))
-#' p + geom_point(data = df3, colour="red", size = 2)
-#'
+#' # If you combine a facetted dataset with a dataset that lacks those
+#' # facetting variables, the data will be repeated across the missing
+#' # combinations:
+#' df <- data.frame(displ = mean(mpg$displ), cty = mean(mpg$cty))
+#' p +
+#'   facet_grid(. ~ cyl) +
+#'   geom_point(data = df, colour = "red", size = 2)
 #'
+#' # Free scales -------------------------------------------------------
 #' # You can also choose whether the scales should be constant
 #' # across all panels (the default), or whether they should be allowed
 #' # to vary
-#' mt <- ggplot(mtcars, aes(mpg, wt, colour = factor(cyl))) + geom_point()
+#' mt <- ggplot(mtcars, aes(mpg, wt, colour = factor(cyl))) +
+#'   geom_point()
 #'
 #' mt + facet_grid(. ~ cyl, scales = "free")
+#'
 #' # If scales and space are free, then the mapping between position
-#' # and values in the data will be the same across all panels
-#' mt + facet_grid(. ~ cyl, scales = "free", space = "free")
+#' # and values in the data will be the same across all panels. This
+#' # is particularly useful for categorical axes
+#' ggplot(mpg, aes(drv, model)) +
+#'   geom_point() +
+#'   facet_grid(manufacturer ~ ., scales = "free", space = "free") +
+#'   theme(strip.text.y = element_text(angle = 0))
 #'
-#' mt + facet_grid(vs ~ am, scales = "free")
-#' mt + facet_grid(vs ~ am, scales = "free_x")
-#' mt + facet_grid(vs ~ am, scales = "free_y")
-#' mt + facet_grid(vs ~ am, scales = "free", space="free")
-#' mt + facet_grid(vs ~ am, scales = "free", space="free_x")
-#' mt + facet_grid(vs ~ am, scales = "free", space="free_y")
+#' # Facet labels ------------------------------------------------------
+#' p <- ggplot(mtcars, aes(wt, mpg)) + geom_point()
+#' p
 #'
-#' # You may need to set your own breaks for consistent display:
-#' mt + facet_grid(. ~ cyl, scales = "free_x", space="free") +
-#'   scale_x_continuous(breaks = seq(10, 36, by = 2))
-#' # Adding scale limits override free scales:
-#' last_plot() + xlim(10, 15)
+#' # label_both() displays both variable name and value
+#' p + facet_grid(vs ~ cyl, labeller = label_both)
 #'
-#' # Free scales are particularly useful for categorical variables
-#' qplot(cty, model, data=mpg) +
-#'   facet_grid(manufacturer ~ ., scales = "free", space = "free")
-#' # particularly when you reorder factor levels
-#' mpg <- within(mpg, {
-#'   model <- reorder(model, cty)
-#'   manufacturer <- reorder(manufacturer, cty)
-#' })
-#' last_plot() %+% mpg + theme(strip.text.y = element_text())
-#'
-#' # Use as.table to to control direction of horizontal facets, TRUE by default
-#' h <- ggplot(mtcars, aes(x = mpg, y = wt)) + geom_point()
-#' h + facet_grid(cyl ~ vs)
-#' h + facet_grid(cyl ~ vs, as.table = FALSE)
-#'
-#' # Use labeller to control facet labels, label_value is default
-#' h + facet_grid(cyl ~ vs, labeller = label_both)
-#' # Using label_parsed, see ?plotmath for more options
+#' # label_parsed() parses text into mathematical expressions, see ?plotmath
 #' mtcars$cyl2 <- factor(mtcars$cyl, labels = c("alpha", "beta", "sqrt(x, y)"))
-#' k <- qplot(wt, mpg, data = mtcars)
-#' k + facet_grid(. ~ cyl2)
-#' k + facet_grid(. ~ cyl2, labeller = label_parsed)
-#' # For label_bquote the label value is x.
-#' p <- qplot(wt, mpg, data = mtcars)
-#' p + facet_grid(. ~ vs, labeller = label_bquote(alpha ^ .(x)))
-#' p + facet_grid(. ~ vs, labeller = label_bquote(.(x) ^ .(x)))
+#' ggplot(mtcars, aes(wt, mpg)) +
+#'   geom_point() +
+#'   facet_grid(. ~ cyl2, labeller = label_parsed)
 #'
+#' # label_bquote() makes it easy to construct math expressions
+#' p + facet_grid(. ~ vs, labeller = label_bquote(cols = alpha ^ .(vs)))
+#'
+#' # The facet strips can be displayed near the axes with switch
+#' data <- transform(mtcars,
+#'   am = factor(am, levels = 0:1, c("Automatic", "Manual")),
+#'   gear = factor(gear, levels = 3:5, labels = c("Three", "Four", "Five"))
+#' )
+#' p <- ggplot(data, aes(mpg, disp)) + geom_point()
+#' p + facet_grid(am ~ gear, switch = "both")
+#' # It looks better without boxes around the strips
+#' p + facet_grid(am ~ gear, switch = "both") +
+#'   theme(strip.background = element_blank())
+#'
+#' # Margins ----------------------------------------------------------
+#' \donttest{
 #' # Margins can be specified by logically (all yes or all no) or by specific
 #' # variables as (character) variable names
 #' mg <- ggplot(mtcars, aes(x = mpg, y = wt)) + geom_point()
@@ -150,7 +121,8 @@
 #' mg + facet_grid(vs + am ~ gear, margins = "gear")
 #' mg + facet_grid(vs + am ~ gear, margins = c("gear", "am"))
 #' }
-facet_grid <- function(facets, margins = FALSE, scales = "fixed", space = "fixed", shrink = TRUE, labeller = "label_value", as.table = TRUE, drop = TRUE) {
+#' @importFrom plyr as.quoted
+facet_grid <- function(facets, margins = FALSE, scales = "fixed", space = "fixed", shrink = TRUE, labeller = "label_value", as.table = TRUE, switch = NULL, drop = TRUE) {
   scales <- match.arg(scales, c("fixed", "free_x", "free_y", "free"))
   free <- list(
     x = any(scales %in% c("free_x", "free")),
@@ -166,11 +138,11 @@ facet_grid <- function(facets, margins = FALSE, scales = "fixed", space = "fixed
   # Facets can either be a formula, a string, or a list of things to be
   # convert to quoted
   if (is.character(facets)) {
-    facets <- as.formula(facets)
+    facets <- stats::as.formula(facets)
   }
   if (is.formula(facets)) {
-    lhs <- function(x) if(length(x) == 2) NULL else x[-3]
-    rhs <- function(x) if(length(x) == 2) x else x[-2]
+    lhs <- function(x) if (length(x) == 2) NULL else x[-3]
+    rhs <- function(x) if (length(x) == 2) x else x[-2]
 
     rows <- as.quoted(lhs(facets))
     rows <- rows[!sapply(rows, identical, as.name("."))]
@@ -185,10 +157,13 @@ facet_grid <- function(facets, margins = FALSE, scales = "fixed", space = "fixed
     stop("Must specify at least one variable to facet by", call. = FALSE)
   }
 
+  # Check for deprecated labellers
+  labeller <- check_labeller(labeller)
+
   facet(
     rows = rows, cols = cols, margins = margins, shrink = shrink,
-    free = free, space_free = space_free,
-    labeller = labeller, as.table = as.table, drop = drop,
+    free = free, space_free = space_free, labeller = labeller,
+    as.table = as.table, switch = switch, drop = drop,
     subclass = "grid"
   )
 }
@@ -226,17 +201,85 @@ facet_render.grid <- function(facet, panel, coord, theme, geom_grobs) {
   strips$r$heights <- panels$heights
   strips$t$widths <- panels$widths
 
+  # Check if switch is consistent with grid layout
+  switch_x <- !is.null(facet$switch) && facet$switch %in% c("both", "x")
+  switch_y <- !is.null(facet$switch) && facet$switch %in% c("both", "y")
+  if (switch_x && length(strips$t) == 0) {
+    facet$switch <- if (facet$switch == "both") "y" else NULL
+    switch_x <- FALSE
+    warning("Cannot switch x axis strips as they do not exist", call. = FALSE)
+  }
+  if (switch_y && length(strips$r) == 0) {
+    facet$switch <- if (facet$switch == "both") "x" else NULL
+    switch_y <- FALSE
+    warning("Cannot switch y axis strips as they do not exist", call. = FALSE)
+  }
+
+
   # Combine components into complete plot
-  top <- strips$t
-  top <- gtable_add_cols(top, strips$r$widths)
-  top <- gtable_add_cols(top, axes$l$widths, pos = 0)
+  if (is.null(facet$switch)) {
+    top <- strips$t
+    top <- gtable_add_cols(top, strips$r$widths)
+    top <- gtable_add_cols(top, axes$l$widths, pos = 0)
 
-  center <- cbind(axes$l, panels, strips$r, z = c(2, 1, 3))
-  bottom <- axes$b
-  bottom <- gtable_add_cols(bottom, strips$r$widths)
-  bottom <- gtable_add_cols(bottom, axes$l$widths, pos = 0)
+    center <- cbind(axes$l, panels, strips$r, z = c(2, 1, 3))
+    bottom <- axes$b
+    bottom <- gtable_add_cols(bottom, strips$r$widths)
+    bottom <- gtable_add_cols(bottom, axes$l$widths, pos = 0)
+
+    complete <- rbind(top, center, bottom, z = c(1, 2, 3))
+
+  } else {
+    # Add padding between the switched strips and the axes
+    padding <- convertUnit(theme$strip.switch.pad.grid, "cm")
+
+    if (switch_x) {
+      t_heights <- c(padding, strips$t$heights)
+      gt_t <- gtable(widths = strips$t$widths, heights = unit(t_heights, "cm"))
+      gt_t <- gtable_add_grob(gt_t, strips$t, name = strips$t$name, clip = "off",
+        t = 1, l = 1, b = -1, r = -1)
+    }
+    if (switch_y) {
+      r_widths <- c(strips$r$widths, padding)
+      gt_r <- gtable(widths = unit(r_widths, "cm"), heights = strips$r$heights)
+      gt_r <- gtable_add_grob(gt_r, strips$r, name = strips$r$name, clip = "off",
+        t = 1, l = 1, b = -1, r = -1)
+    }
+
+    # Combine plot elements according to strip positions
+    if (switch_x && switch_y) {
+      center <- cbind(gt_r, axes$l, panels, z = c(3, 2, 1))
+
+      bottom <- rbind(axes$b, gt_t)
+      bottom <- gtable_add_cols(bottom, axes$l$widths, pos = 0)
+      bottom <- gtable_add_cols(bottom, gt_r$widths, pos = 0)
+
+      complete <- rbind(center, bottom, z = c(1, 2))
+    } else if (switch_x) {
+      center <- cbind(axes$l, panels, strips$r, z = c(2, 1, 3))
+
+      bottom <- rbind(axes$b, gt_t)
+      bottom <- gtable_add_cols(bottom, strips$r$widths)
+      bottom <- gtable_add_cols(bottom, axes$l$widths, pos = 0)
+
+      complete <- rbind(center, bottom, z = c(1, 2))
+    } else if (switch_y) {
+      top <- strips$t
+      top <- gtable_add_cols(top, axes$l$widths, pos = 0)
+      top <- gtable_add_cols(top, gt_r$widths, pos = 0)
+
+      center <- cbind(gt_r, axes$l, panels, z = c(3, 2, 1))
+      bottom <- axes$b
+      bottom <- gtable_add_cols(bottom, axes$l$widths, pos = 0)
+      bottom <- gtable_add_cols(bottom, gt_r$widths, pos = 0)
+
+      complete <- rbind(top, center, bottom, z = c(1, 2, 3))
+    } else {
+      stop("`switch` must be either NULL, 'both', 'x', or 'y'",
+        call. = FALSE)
+    }
+  }
 
-  complete <- rbind(top, center, bottom, z = c(1, 2, 3))
   complete$respect <- panels$respect
   complete$name <- "layout"
   bottom <- axes$b
@@ -249,60 +292,34 @@ facet_strips.grid <- function(facet, panel, theme) {
   col_vars <- unique(panel$layout[names(facet$cols)])
   row_vars <- unique(panel$layout[names(facet$rows)])
 
-  list(
-    r = build_strip(panel, row_vars, facet$labeller, theme, "r"),
-    t = build_strip(panel, col_vars, facet$labeller, theme, "t")
+  # Adding labels metadata, useful for labellers
+  attr(col_vars, "type") <- "cols"
+  attr(col_vars, "facet") <- "grid"
+  attr(row_vars, "type") <- "rows"
+  attr(row_vars, "facet") <- "grid"
+
+  dir <- list(r = "r", t = "t")
+  if (!is.null(facet$switch) && facet$switch %in% c("both", "x")) {
+    dir$t <- "b"
+  }
+  if (!is.null(facet$switch) && facet$switch %in% c("both", "y")) {
+    dir$r <- "l"
+  }
+
+  strips <- list(
+    r = build_strip(panel, row_vars, facet$labeller,
+      theme, dir$r, switch = facet$switch),
+    t = build_strip(panel, col_vars, facet$labeller,
+      theme, dir$t, switch = facet$switch)
   )
-}
 
-build_strip <- function(panel, label_df, labeller, theme, side = "right") {
-  side <- match.arg(side, c("top", "left", "bottom", "right"))
-  horizontal <- side %in% c("top", "bottom")
-  labeller <- match.fun(labeller)
-
-  # No labelling data, so return empty row/col
-  if (empty(label_df)) {
-    if (horizontal) {
-      widths <- unit(rep(0, max(panel$layout$COL)), "null")
-      return(gtable_row_spacer(widths))
+  Map(function(strip, side) {
+    if (side %in% c("t", "b")) {
+      gtable_add_col_space(strip, theme$panel.margin.x %||% theme$panel.margin)
     } else {
-      heights <- unit(rep(0, max(panel$layout$ROW)), "null")
-      return(gtable_col_spacer(heights))
+      gtable_add_row_space(strip, theme$panel.margin.y %||% theme$panel.margin)
     }
-  }
-
-  # Create matrix of labels
-  labels <- matrix(list(), nrow = nrow(label_df), ncol = ncol(label_df))
-  for (i in seq_len(ncol(label_df))) {
-    labels[, i] <- labeller(names(label_df)[i], label_df[, i])
-  }
-
-  # Render as grobs
-  grobs <- apply(labels, c(1,2), ggstrip, theme = theme,
-    horizontal = horizontal)
-
-  # Create layout
-  name <- paste("strip", side, sep = "-")
-  if (horizontal) {
-    grobs <- t(grobs)
-
-    # Each row is as high as the highest and as a wide as the panel
-    row_height <- function(row) max(laply(row, height_cm))
-    heights <- unit(apply(grobs, 1, row_height), "cm")
-    widths <- unit(rep(1, ncol(grobs)), "null")
-  } else {
-    # Each row is wide as the widest and as high as the panel
-    col_width <- function(col) max(laply(col, width_cm))
-    widths <- unit(apply(grobs, 2, col_width), "cm")
-    heights <- unit(rep(1, nrow(grobs)), "null")
-  }
-  strips <- gtable_matrix(name, grobs, heights = heights, widths = widths)
-
-  if (horizontal) {
-    gtable_add_col_space(strips, theme$panel.margin.x %||% theme$panel.margin)
-  } else {
-    gtable_add_row_space(strips, theme$panel.margin.y %||% theme$panel.margin)
-  }
+  }, strips, dir)
 }
 
 #' @export
@@ -311,15 +328,13 @@ facet_axes.grid <- function(facet, panel, coord, theme) {
 
   # Horizontal axes
   cols <- which(panel$layout$ROW == 1)
-  grobs <- lapply(panel$ranges[cols], coord_render_axis_h,
-    coord = coord, theme = theme)
+  grobs <- lapply(panel$ranges[cols], coord$render_axis_h, theme = theme)
   axes$b <- gtable_add_col_space(gtable_row("axis-b", grobs),
     theme$panel.margin.x %||% theme$panel.margin)
 
   # Vertical axes
   rows <- which(panel$layout$COL == 1)
-  grobs <- lapply(panel$ranges[rows], coord_render_axis_v,
-    coord = coord, theme = theme)
+  grobs <- lapply(panel$ranges[rows], coord$render_axis_v, theme = theme)
   axes$l <- gtable_add_row_space(gtable_col("axis-l", grobs),
     theme$panel.margin.y %||% theme$panel.margin)
 
@@ -333,7 +348,7 @@ facet_panels.grid <- function(facet, panel, coord, theme, geom_grobs) {
   # ask the coordinate system if it wants to specify one
   aspect_ratio <- theme$aspect.ratio
   if (is.null(aspect_ratio) && !facet$free$x && !facet$free$y) {
-    aspect_ratio <- coord_aspect(coord, panel$ranges[[1]])
+    aspect_ratio <- coord$aspect(panel$ranges[[1]])
   }
   if (is.null(aspect_ratio)) {
     aspect_ratio <- 1
@@ -348,11 +363,16 @@ facet_panels.grid <- function(facet, panel, coord, theme, geom_grobs) {
   nrow <- max(panel$layout$ROW)
 
   panel_grobs <- lapply(panels, function(i) {
-    fg <- coord_render_fg(coord, panel$range[[i]], theme)
-    bg <- coord_render_bg(coord, panel$range[[i]], theme)
+    fg <- coord$render_fg(panel$ranges[[i]], theme)
+    bg <- coord$render_bg(panel$ranges[[i]], theme)
 
-    geom_grobs <- lapply(geom_grobs, "[[", i)
-    panel_grobs <- c(list(bg), geom_grobs, list(fg))
+    geom_grobs <- lapply(geom_grobs, `[[`, i)
+
+    if (theme$panel.ontop) {
+      panel_grobs <- c(geom_grobs, list(bg), list(fg))
+    } else {
+      panel_grobs <- c(list(bg), geom_grobs, list(fg))
+    }
 
     gTree(children = do.call("gList", panel_grobs))
   })
@@ -367,14 +387,14 @@ facet_panels.grid <- function(facet, panel, coord, theme, geom_grobs) {
   # In general, panel has all information for building facet.
   if (facet$space_free$x) {
     ps <- panel$layout$PANEL[panel$layout$ROW == 1]
-    widths <- vapply(ps, function(i) diff(panel$range[[i]]$x.range), numeric(1))
+    widths <- vapply(ps, function(i) diff(panel$ranges[[i]]$x.range), numeric(1))
     panel_widths <- unit(widths, "null")
   } else {
     panel_widths <- rep(unit(1, "null"), ncol)
   }
   if (facet$space_free$y) {
     ps <- panel$layout$PANEL[panel$layout$COL == 1]
-    heights <- vapply(ps, function(i) diff(panel$range[[i]]$y.range), numeric(1))
+    heights <- vapply(ps, function(i) diff(panel$ranges[[i]]$y.range), numeric(1))
     panel_heights <- unit(heights, "null")
   } else {
     panel_heights <- rep(unit(1 * aspect_ratio, "null"), nrow)
