@@ -1,4 +1,4 @@
-#' Modify a ggplot or theme object by adding on new components.
+#' Add a new component to a ggplot or theme object.
 #'
 #' This operator allows you to add objects to a ggplot or theme object.
 #'
@@ -23,18 +23,22 @@
 #' For theme objects, the \code{+} operator and the \code{\%+replace\%}
 #' can be used to modify elements in themes.
 #'
-#' The \code{+} operator completely replaces elements
-#' with elements from e2.
+#' The \code{+} operator updates the elements of e1 that differ from
+#' elements specified (not NULL) in e2.
+#' Thus this operator can be used to incrementally add or modify attributes
+#' of a ggplot theme.
 #'
-#' In contrast, the \code{\%+replace\%} operator does not replace the
-#' entire element; it only updates element properties which are present
-#' (not NULL) in the second object.
+#' In contrast, the \code{\%+replace\%} operator replaces the
+#' entire element; any element of a theme not specified in e2 will not be
+#' present in the resulting theme (i.e. NULL).
+#' Thus this operator can be used to overwrite an entire theme.
 #'
 #' @examples
-#'
 #' ### Adding objects to a ggplot object
-#' p <- qplot(wt, mpg, colour = hp, data = mtcars)
+#' p <- ggplot(mtcars, aes(wt, mpg, colour = disp)) +
+#'   geom_point()
 #'
+#' p
 #' p + coord_cartesian(ylim = c(0, 40))
 #' p + scale_colour_continuous(breaks = c(100, 300))
 #' p + guides(colour = "colourbar")
@@ -43,7 +47,6 @@
 #' m <- mtcars[1:10, ]
 #' p %+% m
 #'
-#'
 #' ### Adding objects to a theme object
 #' # Compare these results of adding theme objects to other theme objects
 #' add_el <- theme_grey() + theme(text = element_text(family = "Times"))
@@ -54,9 +57,7 @@
 #'
 #' @param e1 An object of class \code{ggplot} or \code{theme}
 #' @param e2 A component to add to \code{e1}
-#'
 #' @export
-#'
 #' @seealso \code{\link{theme}}
 #' @method + gg
 #' @rdname gg-add
@@ -83,46 +84,36 @@ add_ggplot <- function(p, object, objectname) {
     p$data <- object
   } else if (is.theme(object)) {
     p$theme <- update_theme(p$theme, object)
-  } else if (inherits(object, "scale")) {
+  } else if (inherits(object, "Scale")) {
     p$scales$add(object)
-  } else if(inherits(object, "labels")) {
+  } else if (inherits(object, "labels")) {
     p <- update_labels(p, object)
-  } else if(inherits(object, "guides")) {
+  } else if (inherits(object, "guides")) {
     p <- update_guides(p, object)
-  } else if(inherits(object, "uneval")) {
+  } else if (inherits(object, "uneval")) {
       p$mapping <- defaults(object, p$mapping)
 
       labels <- lapply(object, deparse)
       names(labels) <- names(object)
       p <- update_labels(p, labels)
-  } else if (is.coord(object)) {
+  } else if (is.Coord(object)) {
       p$coordinates <- object
       p
   } else if (is.facet(object)) {
       p$facet <- object
       p
-  } else if(is.list(object)) {
+  } else if (is.list(object)) {
     for (o in object) {
       p <- p + o
     }
-  } else if(is.proto(object)) {
-    p <- switch(object$class(),
-      layer  = {
-        p$layers <- append(p$layers, object)
+  } else if (is.layer(object)) {
+    p$layers <- append(p$layers, object)
 
-        # Add any new labels
-        mapping <- make_labels(object$mapping)
-        default <- make_labels(object$stat$default_aes())
-
-        new_labels <- defaults(mapping, default)
-        p$labels <- defaults(p$labels, new_labels)
-        p
-      },
-      coord = {
-        p$coordinates <- object
-        p
-      }
-    )
+    # Add any new labels
+    mapping <- make_labels(object$mapping)
+    default <- make_labels(object$stat$default_aes)
+    new_labels <- defaults(mapping, default)
+    p$labels <- defaults(p$labels, new_labels)
   } else {
     stop("Don't know how to add ", objectname, " to a plot",
       call. = FALSE)
