@@ -1,4 +1,5 @@
-#' @inheritParams stat_identity
+#' @inheritParams layer
+#' @inheritParams geom_point
 #' @section Computed variables:
 #' \describe{
 #'  \item{n}{number of observations at position}
@@ -6,9 +7,12 @@
 #' }
 #' @export
 #' @rdname geom_count
-stat_sum <- function(mapping = NULL, data = NULL, geom = "point",
-                     position = "identity", na.rm = FALSE,
-                     show.legend = NA, inherit.aes = TRUE, ...) {
+stat_sum <- function(mapping = NULL, data = NULL,
+                     geom = "point", position = "identity",
+                     ...,
+                     na.rm = FALSE,
+                     show.legend = NA,
+                     inherit.aes = TRUE) {
   layer(
     data = data,
     mapping = mapping,
