@@ -1,16 +1,9 @@
-#' Summarise y values at every unique x.
+#' Summarise y values at unique/binned x x.
 #'
-#' \code{stat_summary} allows for tremendous flexibilty in the specification
-#' of summary functions. The summary function can either supply individual
-#' summary functions for each of y, ymin and ymax (with \code{fun.y},
-#' \code{fun.ymax}, \code{fun.ymin}), or return a data frame containing any
-#' number of aesthetiics with with \code{fun.data}. All summary functions
-#' are called with a single vector of values, \code{x}.
-#'
-#' A simple vector function is easiest to work with as you can return a single
-#' number, but is somewhat less flexible.  If your summary function operates
-#' on a data.frame it should return a data frame with variables that the geom
-#' can use.
+#' \code{stat_summary} operates on unique \code{x}; \code{stat_summary_bin}
+#' operators on binned \code{x}. They are more flexible versions of
+#' \code{\link{stat_bin}}: instead of just counting, they can compute any
+#' aggregate.
 #'
 #' @section Aesthetics:
 #' \Sexpr[results=rd,stage=build]{ggplot2:::rd_aesthetics("stat", "summary")}
@@ -19,8 +12,12 @@
 #'  \code{\link{geom_linerange}}, \code{\link{geom_crossbar}} for geoms to
 #'  display summarised data
 #' @inheritParams stat_identity
-#' @return a data.frame with additional columns:
-#'   \item{fun.data}{Complete summary function. Should take data frame as
+#' @section Summary functions:
+#' You can either supply summary functions individually (\code{fun.y},
+#' \code{fun.ymax}, \code{fun.ymin}), or as a single function (\code{fun.data}):
+#'
+#' \describe{
+#'   \item{fun.data}{Complete summary function. Should take numeric vector as
 #'      input and return data frame as output}
 #'   \item{fun.ymin}{ymin summary function (should take numeric vector and
 #'     return single number)}
@@ -28,68 +25,63 @@
 #'     single number)}
 #'   \item{fun.ymax}{ymax summary function (should take numeric vector and
 #'     return single number)}
+#' }
+#'
+#' A simple vector function is easiest to work with as you can return a single
+#' number, but is somewhat less flexible. If your summary function computes
+#' multiple values at once (e.g. ymin and ymax), use \code{fun.data}.
+#'
+#' If no aggregation functions are suppled, will default to
+#' \code{\link{mean_se}}.
+#'
+#' @param fun.data A function that is given the complete data and should
+#'   return a data frame with variables \code{ymin}, \code{y}, and \code{ymax}.
+#' @param fun.ymin,fun.y,fun.ymax Alternatively, supply three individual
+#'   functions that are each passed a vector of x's and should return a
+#'   single number.
+#' @param fun.args Optional additional arguments passed on to the functions.
 #' @export
 #' @examples
-#' \donttest{
-#' # Basic operation on a small dataset
-#' d <- qplot(cyl, mpg, data=mtcars)
-#' d + stat_summary(fun.data = "mean_cl_boot", colour = "red")
-#'
-#' p <- qplot(cyl, mpg, data = mtcars, stat="summary", fun.y = "mean")
-#' p
-#' # Don't use ylim to zoom into a summary plot - this throws the
-#' # data away
-#' p + ylim(15, 30)
-#' # Instead use coord_cartesian
-#' p + coord_cartesian(ylim = c(15, 30))
+#' d <- ggplot(mtcars, aes(cyl, mpg)) + geom_point()
+#' d + stat_summary(fun.data = "mean_cl_boot", colour = "red", size = 2)
 #'
 #' # You can supply individual functions to summarise the value at
 #' # each x:
-#'
-#' stat_sum_single <- function(fun, geom="point", ...) {
-#'   stat_summary(fun.y=fun, colour="red", geom=geom, size = 3, ...)
-#' }
-#'
-#' d + stat_sum_single(mean)
-#' d + stat_sum_single(mean, geom="line")
-#' d + stat_sum_single(median)
-#' d + stat_sum_single(sd)
+#' d + stat_summary(fun.y = "median", colour = "red", size = 2, geom = "point")
+#' d + stat_summary(fun.y = "mean", colour = "red", size = 2, geom = "point")
+#' d + aes(colour = factor(vs)) + stat_summary(fun.y = mean, geom="line")
 #'
 #' d + stat_summary(fun.y = mean, fun.ymin = min, fun.ymax = max,
 #'   colour = "red")
 #'
-#' d + aes(colour = factor(vs)) + stat_summary(fun.y = mean, geom="line")
+#' d <- ggplot(diamonds, aes(cut))
+#' d + geom_bar()
+#' d + stat_summary_bin(aes(y = price), fun.y = "mean", geom = "bar")
+#'
+#' \donttest{
+#' # Don't use ylim to zoom into a summary plot - this throws the
+#' # data away
+#' p <- ggplot(mtcars, aes(cyl, mpg)) +
+#'   stat_summary(fun.y = "mean", geom = "point")
+#' p
+#' p + ylim(15, 30)
+#' # Instead use coord_cartesian
+#' p + coord_cartesian(ylim = c(15, 30))
 #'
-#' # Alternatively, you can supply a function that operates on a data.frame.
 #' # A set of useful summary functions is provided from the Hmisc package:
-#'
 #' stat_sum_df <- function(fun, geom="crossbar", ...) {
 #'   stat_summary(fun.data=fun, colour="red", geom=geom, width=0.2, ...)
 #' }
-#'
+#' d <- ggplot(mtcars, aes(cyl, mpg)) + geom_point()
 #' # The crossbar geom needs grouping to be specified when used with
 #' # a continuous x axis.
 #' d + stat_sum_df("mean_cl_boot", mapping = aes(group = cyl))
 #' d + stat_sum_df("mean_sdl", mapping = aes(group = cyl))
-#' d + stat_sum_df("mean_sdl", mult = 1, mapping = aes(group = cyl))
+#' d + stat_sum_df("mean_sdl", fun.args = list(mult = 1), mapping = aes(group = cyl))
 #' d + stat_sum_df("median_hilow", mapping = aes(group = cyl))
 #'
-#' # There are lots of different geoms you can use to display the summaries
-#'
-#' d + stat_sum_df("mean_cl_normal", mapping = aes(group = cyl))
-#' d + stat_sum_df("mean_cl_normal", geom = "errorbar")
-#' d + stat_sum_df("mean_cl_normal", geom = "pointrange")
-#' d + stat_sum_df("mean_cl_normal", geom = "smooth")
-#'
-#' # Summaries are more useful with a bigger data set:
-#' mpg2 <- subset(mpg, cyl != 5L)
-#' m <- ggplot(mpg2, aes(x=cyl, y=hwy)) +
-#'         geom_point() +
-#'         stat_summary(fun.data = "mean_sdl", geom = "linerange",
-#'                      colour = "red", size = 2, mult = 1) +
-#'        xlab("cyl")
-#' m
 #' # An example with highly skewed distributions:
+#' if (require("ggplot2movies")) {
 #' set.seed(596)
 #' mov <- movies[sample(nrow(movies), 1000), ]
 #'  m2 <- ggplot(mov, aes(x= factor(round(rating)), y=votes)) + geom_point()
@@ -110,41 +102,53 @@
 #' # standard errors.
 #' m2 + coord_trans(y="log10")
 #' }
-stat_summary <- function (mapping = NULL, data = NULL, geom = "pointrange", position = "identity", ...) {
-  StatSummary$new(mapping = mapping, data = data, geom = geom, position = position, ...)
+#' }
+stat_summary <- function(mapping = NULL, data = NULL,
+                         geom = "pointrange", position = "identity",
+                         ...,
+                         fun.data = NULL,
+                         fun.y = NULL,
+                         fun.ymax = NULL,
+                         fun.ymin = NULL,
+                         fun.args = list(),
+                         na.rm = FALSE,
+                         show.legend = NA,
+                         inherit.aes = TRUE) {
+  layer(
+    data = data,
+    mapping = mapping,
+    stat = StatSummary,
+    geom = geom,
+    position = position,
+    show.legend = show.legend,
+    inherit.aes = inherit.aes,
+    params = list(
+      fun.data = fun.data,
+      fun.y = fun.y,
+      fun.ymax = fun.ymax,
+      fun.ymin = fun.ymin,
+      fun.args = fun.args,
+      na.rm = na.rm,
+      ...
+    )
+  )
 }
 
-StatSummary <- proto(Stat, {
-  objname <- "summary"
+#' @rdname ggplot2-ggproto
+#' @format NULL
+#' @usage NULL
+#' @export
+StatSummary <- ggproto("StatSummary", Stat,
+  required_aes = c("x", "y"),
 
-  default_geom <- function(.) GeomPointrange
-  required_aes <- c("x", "y")
+  compute_panel = function(data, scales, fun.data = NULL, fun.y = NULL,
+                     fun.ymax = NULL, fun.ymin = NULL, fun.args = list(),
+                     na.rm = FALSE) {
 
-  calculate_groups <- function(., data, scales, fun.data = NULL, fun.y = NULL, fun.ymax = NULL, fun.ymin = NULL, na.rm = FALSE, ...) {
-    data <- remove_missing(data, na.rm, c("x", "y"), name = "stat_summary")
-
-    if (!missing(fun.data)) {
-      # User supplied function that takes complete data frame as input
-      fun.data <- match.fun(fun.data)
-      fun <- function(df, ...) {
-        fun.data(df$y, ...)
-      }
-    } else {
-      # User supplied individual vector functions
-      fs <- compact(list(ymin = fun.ymin, y = fun.y, ymax = fun.ymax))
-
-      fun <- function(df, ...) {
-        res <- llply(fs, function(f) do.call(f, list(df$y, ...)))
-        names(res) <- names(fs)
-        as.data.frame(res)
-      }
-    }
-
-    summarise_by_x(data, fun, ...)
+    fun <- make_summary_fun(fun.data, fun.y, fun.ymax, fun.ymin, fun.args)
+    summarise_by_x(data, fun)
   }
-
-
-})
+)
 
 # Summarise a data.frame by parts
 # Summarise a data frame by unique value of x
@@ -159,11 +163,11 @@ StatSummary <- proto(Stat, {
 # @param other arguments passed on to summary function
 # @keyword internal
 summarise_by_x <- function(data, summary, ...) {
-  summary <- ddply(data, c("group", "x"), summary, ...)
-  unique <- ddply(data, c("group", "x"), uniquecols)
+  summary <- plyr::ddply(data, c("group", "x"), summary, ...)
+  unique <- plyr::ddply(data, c("group", "x"), uniquecols)
   unique$y <- NULL
 
-  merge(summary, unique, by = c("x", "group"))
+  merge(summary, unique, by = c("x", "group"), sort = FALSE)
 }
 
 #' Wrap up a selection of summary functions from Hmisc to make it easy to use
@@ -180,11 +184,15 @@ summarise_by_x <- function(data, summary, ...) {
 NULL
 
 wrap_hmisc <- function(fun) {
-  function(x, ...) {
-    try_require("Hmisc")
 
-    result <- safe.call(fun, list(x = x, ...))
-    rename(
+  function(x, ...) {
+    if (!requireNamespace("Hmisc", quietly = TRUE))
+      stop("Hmisc package required for this function", call. = FALSE)
+
+    fun <- getExportedValue("Hmisc", fun)
+    result <- do.call(fun, list(x = quote(x), ...))
+
+    plyr::rename(
       data.frame(t(result)),
       c(Median = "y", Mean = "y", Lower = "ymin", Upper = "ymax"),
       warn_missing = FALSE
@@ -211,8 +219,8 @@ median_hilow <- wrap_hmisc("smedian.hilow")
 #' @seealso for use with \code{\link{stat_summary}}
 #' @export
 mean_se <- function(x, mult = 1) {
-  x <- na.omit(x)
-  se <- mult * sqrt(var(x) / length(x))
+  x <- stats::na.omit(x)
+  se <- mult * sqrt(stats::var(x) / length(x))
   mean <- mean(x)
   data.frame(y = mean, ymin = mean - se, ymax = mean + se)
 }
