@@ -1,3 +1,9 @@
+#' @export
+grid::unit
+
+#' @export
+grid::arrow
+
 # Name ggplot grid object
 # Convenience function to name grid objects
 #
@@ -10,10 +16,10 @@ ggname <- function(prefix, grob) {
 width_cm <- function(x) {
   if (is.grob(x)) {
     convertWidth(grobWidth(x), "cm", TRUE)
-  } else if (is.list(x)) {
-    vapply(x, width_cm, numeric(1))
   } else if (is.unit(x)) {
     convertWidth(x, "cm", TRUE)
+  } else if (is.list(x)) {
+    vapply(x, width_cm, numeric(1))
   } else {
     stop("Unknown input")
   }
@@ -21,10 +27,10 @@ width_cm <- function(x) {
 height_cm <- function(x) {
   if (is.grob(x)) {
     convertWidth(grobHeight(x), "cm", TRUE)
-  } else if (is.list(x)) {
-    vapply(x, height_cm, numeric(1))
   } else if (is.unit(x)) {
     convertHeight(x, "cm", TRUE)
+  } else if (is.list(x)) {
+    vapply(x, height_cm, numeric(1))
   } else {
     stop("Unknown input")
   }
